@@ -33,12 +33,6 @@
 #include "corpus/corpus.h"
 #include "error.h"
 
-typedef enum
-{
-	VDAL=1, /* Sort by Violations, Disjunct cost, And cost, Link cost */
-	CORPUS, /* Sort by Corpus cost */
-}  Cost_Model_type;
-
 struct Cost_Model_s
 {
 	Cost_Model_type type;
@@ -47,8 +41,8 @@ struct Cost_Model_s
 
 struct Resources_s
 {
-	int    max_parse_time;
-	size_t max_memory;
+	int    max_parse_time;  /* in seconds */
+	size_t max_memory;      /* in bytes */
 	double time_when_parse_started;
 	size_t space_when_parse_started;
 	double when_created;
@@ -206,6 +200,7 @@ struct And_data_s
 struct Parse_info_struct
 {
 	int            x_table_size;
+	int            log2_x_table_size;
 	X_table_connector ** x_table;
 	Parse_set *    parse_set;
 	int            N_words;
@@ -255,38 +250,43 @@ struct Sentence_s
 	String_set *   string_set;  /* used for word names, not connectors */
 	And_data       and_data;    /* used to keep track of fat disjuncts */
 	char  q_pruned_rules;       /* don't prune rules more than once in p.p. */
-	int   post_quote[MAX_SENTENCE];
+	int   post_quote[MAX_SENTENCE]; /* Used only by tokenizer. */
 
 	analyze_context_t * analyze_ctxt; /* private state  used for analyzing */
 	count_context_t * count_ctxt; /* private state info used for counting */
 	match_context_t * match_ctxt; /* private state info used for matching */
 	/* thread-safe random number state */
 	unsigned int rand_state;
+
+	/* Hook for the SAT solver */
+	void *hook;
 };
 
 /*********************************************************
-*
-* Post processing
-*
-**********************************************************/
+ *
+ * Post processing
+ * XXX FIXME: most of these structures should not be in the
+ * public API; they're here only because they're tangled into
+ * sublinkages, which will go away when fat-links are removed.
+ **********************************************************/
 
 struct Domain_s
 {
 	const char *   string;
-	int            size;
 	List_o_links * lol;
-	int            start_link;  /* the link that started this domain */
-	int            type;        /* one letter name */
 	DTreeLeaf *    child;
 	Domain *       parent;
+	int            size;
+	int            start_link;  /* the link that started this domain */
+	char           type;        /* one letter name */
 };
 
 
 struct DTreeLeaf_s
 {
 	Domain *    parent;
-	int         link;
 	DTreeLeaf * next;
+	int         link;
 };
 
 struct PP_data_s
@@ -322,10 +322,10 @@ struct Postprocessor_s
 
 
 /*********************************************************
-*
-* Linkages
-*
-**********************************************************/
+ *
+ * Linkages
+ *
+ **********************************************************/
 
 struct Sublinkage_s
 {
