@@ -132,7 +132,7 @@ int parse_options_delete(Parse_Options  opts)
 	return 0;
 }
 
-void parse_options_set_cost_model_type(Parse_Options opts, int cm)
+void parse_options_set_cost_model_type(Parse_Options opts, Cost_Model_type cm)
 {
 	switch(cm) {
 	case VDAL:
@@ -152,7 +152,7 @@ void parse_options_set_cost_model_type(Parse_Options opts, int cm)
 	}
 }
 
-int  parse_options_get_cost_model_type(Parse_Options opts)
+Cost_Model_type parse_options_get_cost_model_type(Parse_Options opts)
 {
 	return opts->cost_model.type;
 }
@@ -575,7 +575,6 @@ dictionary_six(const char * lang, const char * dict_name,
 	Dict_node *dict_node;
 
 	init_memusage();
-	init_randtable();
 
 	dict = (Dictionary) xalloc(sizeof(struct Dictionary_s));
 	memset(dict, 0, sizeof(struct Dictionary_s));
@@ -1140,6 +1139,7 @@ int sentence_split(Sentence sent, Parse_Options opts)
 void sentence_delete(Sentence sent)
 {
 	if (!sent) return;
+	sat_sentence_delete(sent);
 	/* free_andlists(sent); */
 	free_sentence_disjuncts(sent);
 	free_sentence_expressions(sent);
@@ -1200,27 +1200,40 @@ int sentence_num_linkages_post_processed(Sentence sent) {
 
 int sentence_num_violations(Sentence sent, int i) {
 	if (!sent) return 0;
+
+	/* The sat solver (currently) fails to fill in link_info */
+	if (!sent->link_info) return 0;
 	return sent->link_info[i].N_violations;
 }
 
 int sentence_and_cost(Sentence sent, int i) {
 	if (!sent) return 0;
+
+	/* The sat solver (currently) fails to fill in link_info */
+	if (!sent->link_info) return 0;
 	return sent->link_info[i].and_cost;
 }
 
 int sentence_disjunct_cost(Sentence sent, int i) {
 	if (!sent) return 0;
+
+	/* The sat solver (currently) fails to fill in link_info */
+	if (!sent->link_info) return 0;
 	return sent->link_info[i].disjunct_cost;
 }
 
 int sentence_link_cost(Sentence sent, int i) {
 	if (!sent) return 0;
+
+	/* The sat solver (currently) fails to fill in link_info */
+	if (!sent->link_info) return 0;
 	return sent->link_info[i].link_cost;
 }
 
 int sentence_nth_word_has_disjunction(Sentence sent, int i)
 {
 	if (!sent) return 0;
+	prt_error("Warning: sentence_nth_word_has_disjunction() is deprecated!\n");
 	return (sent->parse_info->chosen_disjuncts[i] != NULL);
 }
 
@@ -1228,12 +1241,11 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 {
 	int nl;
 
-	init_count(sent);
-
 	/* Build lists of disjuncts */
 	prepare_to_parse(sent, opts);
 
 	init_fast_matcher(sent);
+	init_count(sent);
 
 	/* A parse set may have been already been built for this sentence,
 	 * if it was previously parsed.  If so we free it up before
@@ -1271,8 +1283,13 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 
 		post_process_linkages(sent, opts);
 		if (sent->num_valid_linkages > 0) break;
+
+		/* If we are here, then no valid linakges were found.
+		 * If there was a parse overflow, give up now. */
+		if (PARSE_NUM_OVERFLOW < total) break;
 	}
 
+	free_count(sent);
 	free_fast_matcher(sent);
 }
 
@@ -1337,6 +1354,11 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 {
 	Linkage linkage;
 
+	if (opts->use_sat_solver)
+	{
+		return sat_create_linkage(k, sent, opts);
+	}
+
 	if ((k >= sent->num_linkages_post_processed) || (k < 0)) return NULL;
 
 	/* Using exalloc since this is external to the parser itself. */
@@ -1373,7 +1395,8 @@ Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
 	return linkage;
 }
 
-int linkage_get_current_sublinkage(Linkage linkage) { 
+int linkage_get_current_sublinkage(const Linkage linkage)
+{ 
 	return linkage->current; 
 } 
 
@@ -1548,22 +1571,23 @@ int linkage_compute_union(Linkage linkage)
 	return 1;
 }
 
-int linkage_get_num_sublinkages(Linkage linkage) {
+int linkage_get_num_sublinkages(const Linkage linkage)
+{
 	return linkage->num_sublinkages;
 }
 
-int linkage_get_num_words(Linkage linkage)
+int linkage_get_num_words(const Linkage linkage)
 {
 	return linkage->num_words;
 }
 
-int linkage_get_num_links(Linkage linkage)
+int linkage_get_num_links(const Linkage linkage)
 {
 	int current = linkage->current;
 	return linkage->sublinkage[current].num_links;
 }
 
-static inline int verify_link_index(Linkage linkage, int index)
+static inline int verify_link_index(const Linkage linkage, int index)
 {
 	if ((index < 0) ||
 		(index >= linkage->sublinkage[linkage->current].num_links))
@@ -1573,7 +1597,7 @@ static inline int verify_link_index(Linkage linkage, int index)
 	return 1;
 }
 
-int linkage_get_link_length(Linkage linkage, int index)
+int linkage_get_link_length(const Linkage linkage, int index)
 {
 	Link *link;
 	int word_has_link[MAX_SENTENCE];
@@ -1600,7 +1624,7 @@ int linkage_get_link_length(Linkage linkage, int index)
 	return length;
 }
 
-int linkage_get_link_lword(Linkage linkage, int index)
+int linkage_get_link_lword(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return -1;
@@ -1608,7 +1632,7 @@ int linkage_get_link_lword(Linkage linkage, int index)
 	return link->l;
 }
 
-int linkage_get_link_rword(Linkage linkage, int index)
+int linkage_get_link_rword(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return -1;
@@ -1616,7 +1640,7 @@ int linkage_get_link_rword(Linkage linkage, int index)
 	return link->r;
 }
 
-const char * linkage_get_link_label(Linkage linkage, int index)
+const char * linkage_get_link_label(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -1624,7 +1648,7 @@ const char * linkage_get_link_label(Linkage linkage, int index)
 	return link->name;
 }
 
-const char * linkage_get_link_llabel(Linkage linkage, int index)
+const char * linkage_get_link_llabel(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -1632,7 +1656,7 @@ const char * linkage_get_link_llabel(Linkage linkage, int index)
 	return link->lc->string;
 }
 
-const char * linkage_get_link_rlabel(Linkage linkage, int index)
+const char * linkage_get_link_rlabel(const Linkage linkage, int index)
 {
 	Link *link;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -1640,17 +1664,17 @@ const char * linkage_get_link_rlabel(Linkage linkage, int index)
 	return link->rc->string;
 }
 
-const char ** linkage_get_words(Linkage linkage)
+const char ** linkage_get_words(const Linkage linkage)
 {
 	return linkage->word;
 }
 
-Sentence linkage_get_sentence(Linkage linkage)
+Sentence linkage_get_sentence(const Linkage linkage)
 {
 	return linkage->sent;
 }
 
-const char * linkage_get_disjunct_str(Linkage linkage, int w)
+const char * linkage_get_disjunct_str(const Linkage linkage, int w)
 {
 	Disjunct *dj;
 
@@ -1666,7 +1690,7 @@ const char * linkage_get_disjunct_str(Linkage linkage, int w)
 	return linkage->info->disjunct_list_str[w];
 }
 
-double linkage_get_disjunct_cost(Linkage linkage, int w)
+double linkage_get_disjunct_cost(const Linkage linkage, int w)
 {
 	Disjunct *dj = linkage->sent->parse_info->chosen_disjuncts[w];
 
@@ -1675,7 +1699,7 @@ double linkage_get_disjunct_cost(Linkage linkage, int w)
 	return 0.0;
 }
 
-double linkage_get_disjunct_corpus_score(Linkage linkage, int w)
+double linkage_get_disjunct_corpus_score(const Linkage linkage, int w)
 {
 	Disjunct *dj = linkage->sent->parse_info->chosen_disjuncts[w];
 
@@ -1685,42 +1709,54 @@ double linkage_get_disjunct_corpus_score(Linkage linkage, int w)
 	return lg_corpus_disjunct_score(linkage, w); 
 }
 
-const char * linkage_get_word(Linkage linkage, int w)
+const char * linkage_get_word(const Linkage linkage, int w)
 {
 	return linkage->word[w];
 }
 
-int linkage_unused_word_cost(Linkage linkage)
+int linkage_unused_word_cost(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return 0;
 	return linkage->info->unused_word_cost;
 }
 
-int linkage_disjunct_cost(Linkage linkage)
+int linkage_disjunct_cost(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return 0;
 	return (int) floorf(linkage->info->disjunct_cost);
 }
 
-int linkage_is_fat(Linkage linkage)
+int linkage_is_fat(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return 0;
 	return linkage->info->fat;
 }
 
 int linkage_and_cost(Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return 0;
 	return linkage->info->and_cost;
 }
 
-int linkage_link_cost(Linkage linkage)
+int linkage_link_cost(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return 0;
 	return linkage->info->link_cost;
 }
 
-double linkage_corpus_cost(Linkage linkage)
+double linkage_corpus_cost(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return 0.0;
 	return linkage->info->corpus_cost;
 }
 
-int linkage_get_link_num_domains(Linkage linkage, int index)
+int linkage_get_link_num_domains(const Linkage linkage, int index)
 {
 	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return -1;
@@ -1728,7 +1764,7 @@ int linkage_get_link_num_domains(Linkage linkage, int index)
 	return pp_info->num_domains;
 }
 
-const char ** linkage_get_link_domain_names(Linkage linkage, int index)
+const char ** linkage_get_link_domain_names(const Linkage linkage, int index)
 {
 	PP_info *pp_info;
 	if (!verify_link_index(linkage, index)) return NULL;
@@ -1736,23 +1772,29 @@ const char ** linkage_get_link_domain_names(Linkage linkage, int index)
 	return pp_info->domain_name;
 }
 
-const char * linkage_get_violation_name(Linkage linkage)
+const char * linkage_get_violation_name(const Linkage linkage)
 {
 	return linkage->sublinkage[linkage->current].violation;
 }
 
-int linkage_is_canonical(Linkage linkage)
+int linkage_is_canonical(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return TRUE;
 	return linkage->info->canonical;
 }
 
-int linkage_is_improper(Linkage linkage)
+int linkage_is_improper(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return FALSE;
 	return linkage->info->improper_fat_linkage;
 }
 
-int linkage_has_inconsistent_domains(Linkage linkage)
+int linkage_has_inconsistent_domains(const Linkage linkage)
 {
+	/* The sat solver (currently) fails to fill in info */
+	if (!linkage->info) return FALSE;
 	return linkage->info->inconsistent_domains;
 }
 
