@@ -16,6 +16,7 @@
 #include <link-grammar/api.h>
 #include "error.h"
 #include "constituents.h"
+#include "post-process.h"
 
 #define MAXCONSTITUENTS 8192
 #define MAXSUBL 16
@@ -27,20 +28,21 @@ typedef enum {NONE, STYPE, PTYPE, QTYPE, QDTYPE} WType;
 
 typedef struct
 {
-	int left;
-	int right;
 	const char * type;
-	char domain_type;
 	const char * start_link;
-	int start_num;
+	int left;      /* leftmost word */
+	int right;     /* rightmost word */
 	int subl;
 	int canon;
 	int valid;
+	char domain_type;
 #ifdef AUX_CODE_IS_DEAD
 	/* The only code that actually sets aux to a non-zero value is code
 	 * followed by code that zets it to zero. -- its dead code, and so 
-	 * aux is never actually used. Comment this code out.
+	 * aux is never actually used. Comment this code out. It was here
+	 * TreeBank I compatibility, and isn't used in TreeBank II.
 	 */
+	int start_num; /* starting link */
 	int aux;	
 	/* 0: it's an ordinary VP (or other type);
 	 * 1: it's an AUX, don't print it;
@@ -137,6 +139,20 @@ static void print_constituent(con_context_t *ctxt, Linkage linkage, int c)
  *                      sublinkage
  *********************************************************/
 
+typedef enum
+{
+	CASE_S=1,
+	CASE_UNUSED=2,  /* XXX not used anywhere... */
+	CASE_REL_CLAUSE=3,
+	CASE_APPOS=4,
+	CASE_OPENER=5,
+	CASE_PPOPEN=6,
+	CASE_SVINV=7,
+	CASE_PART_MOD=8,
+	CASE_PART_OPEN=9,
+
+} case_type;
+
 /**
  * This function looks for constituents of type ctype1. Say it finds
  * one, call it c1. It searches for the next larger constituent of
@@ -146,7 +162,7 @@ static void print_constituent(con_context_t *ctxt, Linkage linkage, int c)
 static int gen_comp(con_context_t *ctxt, Linkage linkage,
                     int numcon_total, int numcon_subl,
 					     const char * ctype1, const char * ctype2,
-                    const char * ctype3, int x)
+                    const char * ctype3, case_type x)
 {
 	int w, w2, w3, c, c1, c2, done;
 	c = numcon_total + numcon_subl;
@@ -154,12 +170,14 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 	for (c1=numcon_total; c1<numcon_total + numcon_subl; c1++)
 	{
 		/* If ctype1 is NP, it has to be an appositive to continue */
-		if ((x==4) && (post_process_match("MX#*", ctxt->constituent[c1].start_link)==0))
+		if ((x==CASE_APPOS) && (post_process_match("MX#*", ctxt->constituent[c1].start_link)==0))
 			continue;
 
+#ifdef REVIVE_DEAD_CODE
 		/* If ctype1 is X, and domain_type is t, it's an infinitive - skip it */
-		if ((x==2) && (ctxt->constituent[c1].domain_type=='t'))
+		if ((x==CASE_UNUSED) && (ctxt->constituent[c1].domain_type=='t'))
 			continue;
+#endif /* REVIVE_DEAD_CODE */
 
 		/* If it's domain-type z, it's a subject-relative clause;
 		   the VP doesn't need an NP */
@@ -168,7 +186,10 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 
 		/* If ctype1 is X or VP, and it's not started by an S, don't generate an NP
 		 (Neither of the two previous checks are necessary now, right?) */
-		if ((x==1 || x==2) &&
+#ifdef REVIVE_DEAD_CODE
+		/* use this ... if ((x==CASE_S || x==CASE_UNUSED) && */
+#endif /* REVIVE_DEAD_CODE */
+		if ((x==CASE_S) &&
 			(((post_process_match("S", ctxt->constituent[c1].start_link) == 0) &&
 			  (post_process_match("SX", ctxt->constituent[c1].start_link) == 0) &&
 			  (post_process_match("SF", ctxt->constituent[c1].start_link) == 0)) ||
@@ -176,7 +197,7 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 			continue;
 
 		/* If it's an SBAR (relative clause case), it has to be a relative clause */
-		if ((x==3) &&
+		if ((x==CASE_REL_CLAUSE) &&
 			((post_process_match("Rn", ctxt->constituent[c1].start_link) == 0) &&
 			 (post_process_match("R*", ctxt->constituent[c1].start_link) == 0) &&
 			 (post_process_match("MX#r", ctxt->constituent[c1].start_link) == 0) &&
@@ -185,25 +206,25 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 			continue;
 
 		/* If ctype1 is SBAR (clause opener case), it has to be an f domain */
-		if ((x==5) && (ctxt->constituent[c1].domain_type!='f'))
+		if ((x==CASE_OPENER) && (ctxt->constituent[c1].domain_type!='f'))
 			continue;
 
 		/* If ctype1 is SBAR (pp opener case), it has to be a g domain */
-		if ((x==6) && (ctxt->constituent[c1].domain_type!='g'))
+		if ((x==CASE_PPOPEN) && (ctxt->constituent[c1].domain_type!='g'))
 			continue;
 
 		/* If ctype1 is NP (paraphrase case), it has to be started by an SI */
-		if ((x==7) && (post_process_match("SI", ctxt->constituent[c1].start_link)==0))
+		if ((x==CASE_SVINV) && (post_process_match("SI", ctxt->constituent[c1].start_link)==0))
 			continue;
 
 		/* If ctype1 is VP (participle modifier case), it has to be
 		   started by an Mv or Mg */
-		if ((x==8) && (post_process_match("M", ctxt->constituent[c1].start_link)==0))
+		if ((x==CASE_PART_MOD) && (post_process_match("M", ctxt->constituent[c1].start_link)==0))
 			continue;
 
 		/* If ctype1 is VP (participle opener case), it has
 		   to be started by a COp */
-		if ((x==9) && (post_process_match("COp", ctxt->constituent[c1].start_link)==0))
+		if ((x==CASE_PART_OPEN) && (post_process_match("COp", ctxt->constituent[c1].start_link)==0))
 			continue;
 
 		/* Now start at the bounds of c1, and work outwards until you
@@ -231,7 +252,7 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 					   without going outside the current sublinkage.
 					   (Or substituting right and left as necessary.) */
 
-					if ((x==5) || (x==6) || (x==9)) {
+					if ((x==CASE_OPENER) || (x==CASE_PPOPEN) || (x==CASE_PART_OPEN)) {
 								/* This is the case where c is to the
 								   RIGHT of c1 */
 						w = ctxt->constituent[c1].right+1;
@@ -271,8 +292,10 @@ static int gen_comp(con_context_t *ctxt, Linkage linkage,
 					ctxt->constituent[c].domain_type = 'x';
 					ctxt->constituent[c].start_link =
 						string_set_add("XX", ctxt->phrase_ss);
+#ifdef AUX_CODE_IS_DEAD
 					ctxt->constituent[c].start_num =
 						ctxt->constituent[c1].start_num; /* bogus */
+#endif /* AUX_CODE_IS_DEAD */
 					if (verbosity >= 2)
 					{
 						printf("Larger c found: c %d (%s); ",
@@ -317,7 +340,7 @@ static void adjust_subordinate_clauses(con_context_t *ctxt, Linkage linkage,
 
 	for (c=numcon_total; c<numcon_total + numcon_subl; c++) {
 		if ((post_process_match("MVs", ctxt->constituent[c].start_link) == 1) ||
-			(post_process_match("MVg", ctxt->constituent[c].start_link)==1)) {
+			 (post_process_match("MVg", ctxt->constituent[c].start_link) == 1)) {
 			done=0;
 			for (w2=ctxt->constituent[c].left-1; (done==0) && w2>=0; w2--) {
 				for (c2=numcon_total; c2<numcon_total + numcon_subl; c2++) {
@@ -369,11 +392,10 @@ static int find_next_element(con_context_t *ctxt,
                              int num_elements,
                              int num_lists)
 {
-	int c, a, ok, c2, c3, addedone=0, n;
+	int c, a, ok, c2, c3, addedone=0;
 
 	assert(num_elements <= MAX_ELTS, "Constutent element array overflow!\n");
 
-	n = num_lists;
 	for (c=start+1; c<numcon_total; c++)
 	{
 		constituent_t *cc = &ctxt->constituent[c];
@@ -456,6 +478,9 @@ static int find_next_element(con_context_t *ctxt,
 	return num_lists;
 }
 
+/* When fat-links are finally removed, almost all of the code
+ * for merge_constituents will evaporate away.
+ */
 static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_total)
 {
 	int c1, c2=0, c3, ok, a, n, a2, n2, match, listmatch, a3;
@@ -714,9 +739,10 @@ static int merge_constituents(con_context_t *ctxt, Linkage linkage, int numcon_t
 		ctxt->constituent[c1].domain_type = 'x';
 		ctxt->constituent[c1].valid = 1;
 		ctxt->constituent[c1].start_link = ctxt->constituent[c2].start_link;  /* bogus */
-		ctxt->constituent[c1].start_num = ctxt->constituent[c2].start_num;	/* bogus */
 
 #ifdef AUX_CODE_IS_DEAD /* See comments above */
+		ctxt->constituent[c1].start_num = ctxt->constituent[c2].start_num;	/* bogus */
+
 		/* If a constituent within the andlist is an aux (aux==1),
 		 * set aux for the whole-list constituent to 2, also set
 		 * aux for the smaller constituent to 2, meaning they'll both
@@ -795,10 +821,24 @@ static void generate_misc_word_info(con_context_t * ctxt, Linkage linkage)
 	}
 }
 
+static int new_style_conjunctions(con_context_t *ctxt, Linkage linkage, int numcon_total)
+{
+#ifdef DEBUG
+	int c;
+	for (c = 0; c < numcon_total; c++)
+	{
+		constituent_t *ct = &ctxt->constituent[c];
+		printf("ola %d valid=%d %s start=%s lr=%d %d\n", c, 
+      ct->valid, ct->type, ct->start_link, ct->left, ct->right);
+	}
+#endif
+	return numcon_total;
+}
+
 static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_total)
 {
-	int c, c2, global_leftend_found, adjustment_made,
-		global_rightend_found, lastword, newcon_total = 0;
+	int c, c2, global_leftend_found, adjustment_made;
+	int global_rightend_found, lastword, newcon_total = 0;
 	Sentence sent;
 	sent = linkage_get_sentence(linkage);
 
@@ -938,9 +978,9 @@ static int last_minute_fixes(con_context_t *ctxt, Linkage linkage, int numcon_to
 		print_constituent(ctxt, linkage, c);
 	}
 
-	/* Check once more to see if constituents are nested (checking BETWEEN sublinkages
-	   this time) */
-
+	/* Check once more to see if constituents are nested (checking 
+	 * BETWEEN sublinkages this time). This code probably goes away
+	 * when fat-links get removed. */
 	while (1)
 	{
 		adjustment_made=0;
@@ -1007,7 +1047,8 @@ static void count_words_used(con_context_t *ctxt, Linkage linkage)
 	}
 }
 
-static int add_constituent(con_context_t *ctxt, int c, Linkage linkage, Domain domain,
+static int add_constituent(con_context_t *ctxt, int c, const Linkage linkage,
+                           const Domain *domain,
                            int l, int r, const char * name)
 {
 	int nwords = linkage->num_words-2;
@@ -1019,17 +1060,19 @@ static int add_constituent(con_context_t *ctxt, int c, Linkage linkage, Domain d
 	if (l > nwords) l = nwords;
 	assert(l <= r, "negative constituent length!" );
 
+	ctxt->constituent[c].type = string_set_add(name, ctxt->phrase_ss);
 	ctxt->constituent[c].left = l;
 	ctxt->constituent[c].right = r;
-	ctxt->constituent[c].domain_type = domain.type;
+	ctxt->constituent[c].domain_type = domain->type;
 	ctxt->constituent[c].start_link =
-		linkage_get_link_label(linkage, domain.start_link);
-	ctxt->constituent[c].start_num = domain.start_link;
-	ctxt->constituent[c].type = string_set_add(name, ctxt->phrase_ss);
+		linkage_get_link_label(linkage, domain->start_link);
+#ifdef AUX_CODE_IS_DEAD
+	ctxt->constituent[c].start_num = domain->start_link;
+#endif /* AUX_CODE_IS_DEAD */
 	return c;
 }
 
-static const char * cons_of_domain(Linkage linkage, char domain_type)
+static const char * cons_of_domain(const Linkage linkage, char domain_type)
 {
 	switch (domain_type) {
 	case 'a':
@@ -1085,7 +1128,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 {
 	int d, c, leftlimit, l, leftmost, rightmost, w, c2, numcon_subl = 0, w2;
 	List_o_links * dlink;
-	int rootright, rootleft, adjustment_made;
+	int rootleft, adjustment_made;
 	Sublinkage * subl;
 	const char * name;
 	Domain domain;
@@ -1095,7 +1138,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 	for (d = 0, c = numcon_total; d < subl->pp_data.N_domains; d++, c++)
 	{
 		domain = subl->pp_data.domain_array[d];
-		rootright = linkage_get_link_rword(linkage, domain.start_link);
+		// rootright = linkage_get_link_rword(linkage, domain.start_link);
 		rootleft =  linkage_get_link_lword(linkage, domain.start_link);
 
 		if ((domain.type=='c') ||
@@ -1139,48 +1182,48 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 		}
 
 		c--;
-		c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost,
+		c = add_constituent(ctxt, c, linkage, &domain, leftmost, rightmost,
 						cons_of_domain(linkage, domain.type));
 
 		if (domain.type == 'z')
 		{
-			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "S");
+			c = add_constituent(ctxt, c, linkage, &domain, leftmost, rightmost, "S");
 		}
 		if (domain.type=='c')
 		{
-			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "S");
+			c = add_constituent(ctxt, c, linkage, &domain, leftmost, rightmost, "S");
 		}
 		if ((post_process_match("Ce*", ctxt->constituent[c].start_link)==1) ||
 			(post_process_match("Rn", ctxt->constituent[c].start_link)==1))
 		{
-			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "SBAR");
+			c = add_constituent(ctxt, c, linkage, &domain, leftmost, rightmost, "SBAR");
 		}
 		if ((post_process_match("R*", ctxt->constituent[c].start_link)==1) ||
 			(post_process_match("MX#r", ctxt->constituent[c].start_link)==1))
 		{
 			w = leftmost;
 			if (strcmp(linkage->word[w], ",") == 0) w++;
-			c = add_constituent(ctxt, c, linkage, domain, w, w, "WHNP");
+			c = add_constituent(ctxt, c, linkage, &domain, w, w, "WHNP");
 		}
 		if (post_process_match("Mj", ctxt->constituent[c].start_link) == 1)
 		{
 			w = leftmost;
 			if (strcmp(linkage->word[w], ",") == 0) w++;
-			c = add_constituent(ctxt, c, linkage, domain, w, w+1, "WHPP");
-			c = add_constituent(ctxt, c, linkage, domain, w+1, w+1, "WHNP");
+			c = add_constituent(ctxt, c, linkage, &domain, w, w+1, "WHPP");
+			c = add_constituent(ctxt, c, linkage, &domain, w+1, w+1, "WHNP");
 		}
 		if ((post_process_match("Ss#d", ctxt->constituent[c].start_link)==1) ||
 			(post_process_match("B#d", ctxt->constituent[c].start_link)==1))
 		{
-			c = add_constituent(ctxt, c, linkage, domain, rootleft, rootleft, "WHNP");
-			c = add_constituent(ctxt, c, linkage, domain,
+			c = add_constituent(ctxt, c, linkage, &domain, rootleft, rootleft, "WHNP");
+			c = add_constituent(ctxt, c, linkage, &domain,
 							rootleft, ctxt->constituent[c-1].right, "SBAR");
 		}
 		if (post_process_match("CP", ctxt->constituent[c].start_link)==1)
 		{
 			if (strcmp(linkage->word[leftmost], ",") == 0)
 				ctxt->constituent[c].left++;
-			c = add_constituent(ctxt, c, linkage, domain, 1, linkage->num_words-1, "S");
+			c = add_constituent(ctxt, c, linkage, &domain, 1, linkage->num_words-1, "S");
 		}
 		if ((post_process_match("MVs", ctxt->constituent[c].start_link)==1) ||
 			(domain.type=='f'))
@@ -1190,12 +1233,12 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 				w++;
 			if (strcmp(linkage->word[w], "when") == 0)
 			{
-				c = add_constituent(ctxt, c, linkage, domain, w, w, "WHADVP");
+				c = add_constituent(ctxt, c, linkage, &domain, w, w, "WHADVP");
 			}
 		}
 		if (domain.type=='t')
 		{
-			c = add_constituent(ctxt, c, linkage, domain, leftmost, rightmost, "S");
+			c = add_constituent(ctxt, c, linkage, &domain, leftmost, rightmost, "S");
 		}
 		if ((post_process_match("QI", ctxt->constituent[c].start_link) == 1) ||
 			(post_process_match("Mr", ctxt->constituent[c].start_link) == 1) ||
@@ -1211,7 +1254,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 				name = "WHNP";
 			else
 				assert(0, "Unexpected word type");
-			c = add_constituent(ctxt, c, linkage, domain, w, w, name);
+			c = add_constituent(ctxt, c, linkage, &domain, w, w, name);
 
 			if (ctxt->wordtype[w] == QDTYPE)
 			{
@@ -1223,7 +1266,7 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 
 				/* Adjust the right boundary of previous constituent */
 				ctxt->constituent[c].right = w2 - 1;
-				c = add_constituent(ctxt, c, linkage, domain, w2, rightmost, "S");
+				c = add_constituent(ctxt, c, linkage, &domain, w2, rightmost, "S");
 			}
 		}
 
@@ -1256,39 +1299,39 @@ static int read_constituents_from_domains(con_context_t *ctxt, Linkage linkage,
 	   (This must be done first; the S generated will be needed for
 	   later cases.) */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "SBAR", "S", "S", 5);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "SBAR", "S", "S", CASE_OPENER);
 
 	/* pp opener case */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "PP", "S", "S", 6);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "PP", "S", "S", CASE_PPOPEN);
 
 	/* participle opener case */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "S", "S", "S", 9);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "S", "S", "S", CASE_PART_OPEN);
 
 	/* Subject-phrase case; every main VP generates an S */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "VP", "S", "NP", 1);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "VP", "S", "NP", CASE_S);
 
 	/* Relative clause case; an SBAR generates a complement NP */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "SBAR", "NP", "NP", 3);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "SBAR", "NP", "NP", CASE_REL_CLAUSE);
 
 	/* Participle modifier case */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "VP", "NP", "NP", 8);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "VP", "NP", "NP", CASE_PART_MOD);
 
 	/* PP modifying NP */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "PP", "NP", "NP", 8);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "PP", "NP", "NP", CASE_PART_MOD);
 
 	/* Appositive case */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "NP", "NP", "NP", 4);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "NP", "NP", "NP", CASE_APPOS);
 
 	/* S-V inversion case; an NP generates a complement VP */
 	numcon_subl =
-		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "NP", "SINV", "VP", 7);
+		gen_comp(ctxt, linkage, numcon_total, numcon_subl, "NP", "SINV", "VP", CASE_SVINV);
 
 	adjust_subordinate_clauses(ctxt, linkage, numcon_total, numcon_subl);
 	for (c = numcon_total; c < numcon_total + numcon_subl; c++)
@@ -1522,19 +1565,19 @@ static char * exprint_constituent_structure(con_context_t *ctxt, Linkage linkage
 
 static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 {
-	int num_words;
-	Sentence sent;
 	Postprocessor * pp;
 	int s, numcon_total, numcon_subl, num_subl;
 	char * q;
 
-	sent = linkage_get_sentence(linkage);
 	ctxt->phrase_ss = string_set_create();
 	pp = linkage->sent->dict->constituent_pp;
 	numcon_total = 0;
 
 	count_words_used(ctxt, linkage);
 
+	/* 1 < num_sublinkages only if a parse used fat links. */
+	/* A lot of the complixity here could go away once we
+	 * eliminate fat links for good ...  */
 	num_subl = linkage->num_sublinkages;
 	if (num_subl > MAXSUBL)
 	{
@@ -1548,7 +1591,6 @@ static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 	{
 		linkage_set_current_sublinkage(linkage, s);
 		linkage_post_process(linkage, pp);
-		num_words = linkage_get_num_words(linkage);
 		generate_misc_word_info(ctxt, linkage);
 		numcon_subl = read_constituents_from_domains(ctxt, linkage, numcon_total, s);
 		numcon_total = numcon_total + numcon_subl;
@@ -1569,6 +1611,14 @@ static char * do_print_flat_constituents(con_context_t *ctxt, Linkage linkage)
 		err_msg(&ec, Error, "Error: Too many constituents (d).\n");
 		numcon_total = MAXCONSTITUENTS-1;
 	}
+	numcon_total = new_style_conjunctions(ctxt, linkage, numcon_total);
+	if (MAXCONSTITUENTS <= numcon_total)
+	{
+		err_ctxt ec;
+		ec.sent = linkage->sent;
+		err_msg(&ec, Error, "Error: Too many constituents (n).\n");
+		numcon_total = MAXCONSTITUENTS-1;
+	}
 	numcon_total = last_minute_fixes(ctxt, linkage, numcon_total);
 	if (MAXCONSTITUENTS <= numcon_total)
 	{
@@ -1722,7 +1772,8 @@ static void print_tree(String * cs, int indent, CNode * n, int o1, int o2)
 	append_string(cs, ")");
 }
 
-static int assign_spans(CNode * n, int start) {
+static int assign_spans(CNode * n, int start)
+{
 	int num_words=0;
 	CNode * m=NULL;
 	if (n==NULL) return 0;
