@@ -13,10 +13,17 @@
 #ifndef _LINKINCLUDESH_
 #define _LINKINCLUDESH_
 
+#include <link-grammar/api-types.h>
 #include <link-grammar/link-features.h>
 
 LINK_BEGIN_DECLS
 
+#if  __GNUC__ > 2
+#define GNUC_DEPRECATED __attribute__((deprecated))
+#else
+#define GNUC_DEPRECATED
+#endif
+
 /**********************************************************************
  *
  * System initialization
@@ -57,11 +64,6 @@ link_public_api(void)
 link_public_api(char *)
      dictionary_get_data_dir(void);
 
-link_public_api(int)
-     dictionary_is_past_tense_form(Dictionary dict, const char * str);
-link_public_api(int)
-     dictionary_is_entity(Dictionary dict, const char * str);
-
 /**********************************************************************
  *
  * Functions to manipulate Parse Options
@@ -127,8 +129,8 @@ link_public_api(void)
 link_public_api(int)
      parse_options_get_max_parse_time(Parse_Options opts);
 link_public_api(void)
-     parse_options_set_cost_model_type(Parse_Options opts, int cm);
-link_public_api(int)
+     parse_options_set_cost_model_type(Parse_Options opts, Cost_Model_type cm);
+link_public_api(Cost_Model_type)
      parse_options_get_cost_model_type(Parse_Options opts);
 link_public_api(void)
      parse_options_set_use_fat_links(Parse_Options opts, int use_fat_links);
@@ -280,51 +282,51 @@ typedef struct Linkage_s * Linkage;
 link_public_api(Linkage)
      linkage_create(int index, Sentence sent, Parse_Options opts);
 link_public_api(int) 
-     linkage_get_current_sublinkage(Linkage linkage); 
+     linkage_get_current_sublinkage(const Linkage linkage); 
 link_public_api(int)
      linkage_set_current_sublinkage(Linkage linkage, int index);
 link_public_api(void)
      linkage_delete(Linkage linkage);
 link_public_api(Sentence)
-     linkage_get_sentence(Linkage linkage);
+     linkage_get_sentence(const Linkage linkage);
 link_public_api(int)
-     linkage_get_num_sublinkages(Linkage linkage);
+     linkage_get_num_sublinkages(const Linkage linkage);
 link_public_api(int)
-     linkage_get_num_words(Linkage linkage);
+     linkage_get_num_words(const Linkage linkage);
 link_public_api(int)
-     linkage_get_num_links(Linkage linkage);
+     linkage_get_num_links(const Linkage linkage);
 link_public_api(int)
-     linkage_get_link_lword(Linkage linkage, int index);
+     linkage_get_link_lword(const Linkage linkage, int index);
 link_public_api(int)
-     linkage_get_link_rword(Linkage linkage, int index);
+     linkage_get_link_rword(const Linkage linkage, int index);
 link_public_api(int)
-     linkage_get_link_length(Linkage linkage, int index);
+     linkage_get_link_length(const Linkage linkage, int index);
 link_public_api(const char *)
-     linkage_get_link_label(Linkage linkage, int index);
+     linkage_get_link_label(const Linkage linkage, int index);
 link_public_api(const char *)
-     linkage_get_link_llabel(Linkage linkage, int index);
+     linkage_get_link_llabel(const Linkage linkage, int index);
 link_public_api(const char *)
-     linkage_get_link_rlabel(Linkage linkage, int index);
+     linkage_get_link_rlabel(const Linkage linkage, int index);
 link_public_api(int)
-     linkage_get_link_num_domains(Linkage linkage, int index);
+     linkage_get_link_num_domains(const Linkage linkage, int index);
 link_public_api(const char **)
-     linkage_get_link_domain_names(Linkage linkage, int index);
+     linkage_get_link_domain_names(const Linkage linkage, int index);
 link_public_api(const char **)
-     linkage_get_words(Linkage linkage);
+     linkage_get_words(const Linkage linkage);
 link_public_api(const char *)
-     linkage_get_disjunct_str(Linkage linkage, int w);
+     linkage_get_disjunct_str(const Linkage linkage, int w);
 link_public_api(double)
-     linkage_get_disjunct_cost(Linkage linkage, int w);
+     linkage_get_disjunct_cost(const Linkage linkage, int w);
 link_public_api(double)
-     linkage_get_disjunct_corpus_score(Linkage linkage, int w);
+     linkage_get_disjunct_corpus_score(const Linkage linkage, int w);
 link_public_api(const char *)
-     linkage_get_word(Linkage linkage, int w);
+     linkage_get_word(const Linkage linkage, int w);
 link_public_api(char *)
-     linkage_print_disjuncts(Linkage linkage);
+     linkage_print_disjuncts(const Linkage linkage);
 link_public_api(void)
      linkage_free_disjuncts(char *str);
 link_public_api(char *)
-     linkage_print_links_and_domains(Linkage linkage);
+     linkage_print_links_and_domains(const Linkage linkage);
 link_public_api(void)
      linkage_free_links_and_domains(char *str);
 link_public_api(char *)
@@ -340,31 +342,31 @@ link_public_api(char *)
 link_public_api(void)
      linkage_free_postscript(char * str);
 link_public_api(char *)
-     linkage_print_diagram(Linkage linkage);
+     linkage_print_diagram(const Linkage linkage);
 link_public_api(void)
      linkage_free_diagram(char * str);
 link_public_api(int)
      linkage_compute_union(Linkage linkage);
 link_public_api(int)
-     linkage_is_fat(Linkage linkage);
+     linkage_is_fat(const Linkage linkage);
 link_public_api(int)
-     linkage_unused_word_cost(Linkage linkage);
+     linkage_unused_word_cost(const Linkage linkage);
 link_public_api(int)
-     linkage_disjunct_cost(Linkage linkage);
+     linkage_disjunct_cost(const Linkage linkage);
 link_public_api(int)
-     linkage_and_cost(Linkage linkage);
+     linkage_and_cost(const Linkage linkage);
 link_public_api(int)
-     linkage_link_cost(Linkage linkage);
+     linkage_link_cost(const Linkage linkage);
 link_public_api(double)
-     linkage_corpus_cost(Linkage linkage);
+     linkage_corpus_cost(const Linkage linkage);
 link_public_api(int)
-     linkage_is_canonical(Linkage linkage);
+     linkage_is_canonical(const Linkage linkage);
 link_public_api(int)
-     linkage_is_improper(Linkage linkage);
+     linkage_is_improper(const Linkage linkage);
 link_public_api(int)
-     linkage_has_inconsistent_domains(Linkage linkage);
+     linkage_has_inconsistent_domains(const Linkage linkage);
 link_public_api(const char *)
-     linkage_get_violation_name(Linkage linkage);
+     linkage_get_violation_name(const Linkage linkage);
 
 
 /**********************************************************************
@@ -434,15 +436,21 @@ link_public_api(void)
  *
  ********************************************************/
 
+/* Both are deprecated, exported only for backwards-compat w/Java API. */
+link_public_api(int)
+     dictionary_is_past_tense_form(Dictionary dict, const char * str);
+link_public_api(int)
+     dictionary_is_entity(Dictionary dict, const char * str);
+
 /* Identical to sentence_get_word()
  * XXX TBD: make this go away in Version 5.0. */
 link_public_api(const char *)
-     sentence_get_nth_word(Sentence sent, int i);
+     sentence_get_nth_word(Sentence sent, int i) GNUC_DEPRECATED;
 
 /* Who uses this function, anyway? How did this get exported?
  * XXX TBD: make this go away in Version 5.0. */
 link_public_api(int)
-     sentence_nth_word_has_disjunction(Sentence sent, int i);
+     sentence_nth_word_has_disjunction(Sentence sent, int i) GNUC_DEPRECATED;
 
 /* This is not intended for general use; its specific to the internals
  * of the command-line client.  It was exported by accident.
@@ -453,7 +461,7 @@ link_public_api(int)
 /* These are obsolete, and do nothing.
  * XXX TBD: make these go away in Version 5.0. */
 link_public_api(void)
-     lperror_clear(void);
+     lperror_clear(void) GNUC_DEPRECATED;
 extern link_public_api(int)
      lperrno;
 extern link_public_api(char)
