@@ -25,6 +25,37 @@ const char * linkgrammar_get_version(void)
 	return s;
 }
 
+const char * linkgrammar_get_dict_version(Dictionary dict)
+{
+	static char * ver = NULL;
+	char * p;
+	Dict_node *dn;
+	Exp *e;
+
+	if (ver) return ver;
+
+	/* The newer dictionaries should contain a macro of the form:
+	 * <dictionary-version-number>: V4v6v6+;
+	 * which would indicate dictionary verison 4.6.6
+	 * Older dictionaries contain no version info.
+    */
+	dn = dictionary_lookup_list(dict, "<dictionary-version-number>");
+	if (NULL == dn) return "[unknown]";
+
+	e = dn->exp;
+	ver = strdup(&e->u.string[1]);
+	p = strchr(ver, 'v');
+	while (p)
+	{
+		*p = '.';
+		p = strchr(p+1, 'v');
+	}
+
+	free_lookup_list(dn);
+   return ver;
+}
+
+
 /*
   The dictionary format:
 
@@ -377,6 +408,7 @@ static int check_connector(Dictionary dict, const char * s)
  * dict_order - order two dictionary words in proper sort order.
  * Return zero if the strings match, else return standard 
  * (locale-dependent) UTF8 sort order.
+ * XXX but its not UTF-8 ordered ...! Does this matter ???
  */
 /* verbose version */
 /*
@@ -405,7 +437,7 @@ int dict_order(char *s, char *t)
 static inline int dict_order(const char *s, const char *t)
 {
 	while (*s != '\0' && *s == *t) {s++; t++;}
-	return (((*s == '.')?(1):((*s)<<1))  -  ((*t == '.')?(1):((*t)<<1)));
+	return ((*s == '.')?(1):(*s))  -  ((*t == '.')?(1):(*t));
 }
 
 /**
@@ -419,12 +451,27 @@ static inline int dict_order(const char *s, const char *t)
  * characters.  A "*" matches anything.  Otherwise, replace "."
  * by "\0", and take the difference.  This behavior matches that
  * of the function dict_order().
+ *
+ * Note: wild-cards are not used in the English dictionaries;
+ * nor are they used in the Persian or Arabic dictionaries...
+ * perhaps they are used to handle stemming in some other 
+ * languages ?????? XXX FIXME, these wildcards might just be
+ * old dead code ... 
  */
 static inline int dict_order_wild(const char * s, const char * t)
 {
+	const char *so = s;
 	while((*s != '\0') && (*s == *t)) {s++; t++;}
-	if ((*s == '*') || (*t == '*')) return 0;
-	return (((*s == '.')?('\0'):(*s))  -  ((*t == '.')?('\0'):(*t)));
+
+	if ((*s == '*') || (*t == '*'))
+	{
+		/* Do not allow wild-card as the very first char.
+		 * Basically, allow *.v and *.eq in the Enlgish dict.
+		 */
+		if (s != so) return 0;
+	}
+
+	return (((*s == '.')?(0):(*s)) - ((*t == '.')?(0):(*t)));
 }
 
 /**
@@ -1318,7 +1365,7 @@ static int read_entry(Dictionary dict)
 			return 0;
 		}
 
-		/* if it's a word-file name */
+		/* If it's a word-file name */
 		/* However, be careful to reject "/.v" which is the division symbol
 		 * used in equations (.v means verb-like) */
 		if ((dict->token[0] == '/') && (dict->token[1] != '.'))
@@ -1513,6 +1560,7 @@ static void print_expression_parens(Exp * n, int need_parens)
 void print_expression(Exp * n)
 {
 	print_expression_parens(n, FALSE);
+	printf("\n");
 }
 #endif /* INFIX_NOTATION */
 
